/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package MCview;

import java.io.*;

import java.net.*;

import java.util.*;
import java.awt.Color;


public class PDBfile extends jalview.io.FileParse {
    public Vector chains = new Vector();
    Vector lineArray = new Vector();

    public PDBfile(String[] lines) {
        for (int i = 0; i < lines.length; i++)
            lineArray.addElement(lines[i]);

        noLines = lineArray.size();
        parse();
    }

    public PDBfile(String inFile, String inType) throws IOException {
        super(inFile, inType);

        String line;
        this.lineArray = new Vector();

        BufferedReader dataIn;

        if (inType.equals("File")) {
            dataIn = new BufferedReader(new FileReader(inFile));
        }
        else if(inType.equals("Paste"))
        {
            dataIn = new BufferedReader(new StringReader(inFile));
        }
        else {
            URL url = new URL(inFile);
            this.fileSize = 0;
            dataIn = new BufferedReader(new InputStreamReader(url.openStream()));
        }

        while ((line = dataIn.readLine()) != null) {
            lineArray.addElement(line);
        }

        noLines = lineArray.size();

        parse();
    }

    public void parse() {
        for (int i = 0; i < lineArray.size(); i++) {
            StringTokenizer str = new StringTokenizer(lineArray.elementAt(i)
                                                               .toString());

            if (str.hasMoreTokens()) {
                String inStr = str.nextToken();

                if (inStr.indexOf("ATOM") != -1) {
                    try {
                        Atom tmpatom = new Atom(str);

                        if (findChain(tmpatom.chain) != null) {
                            //   System.out.println("Adding to chain " + tmpatom.chain);
                            findChain(tmpatom.chain).atoms.addElement(tmpatom);
                        } else {
                            PDBChain tmpchain = new PDBChain(tmpatom.chain);
                            chains.addElement(tmpchain);
                            tmpchain.atoms.addElement(tmpatom);
                        }
                    } catch (NumberFormatException e) {
                        System.err.println("Caught" + e);
                        System.err.println("Record not added to PDB model:" +
                            lineArray.elementAt(i).toString());
                    }
                }
            }
        }

        makeResidueList();
        makeCaBondList();
    }

    public void makeResidueList() {
        for (int i = 0; i < chains.size(); i++) {
            ((PDBChain) chains.elementAt(i)).makeResidueList();
        }
    }

    public void makeCaBondList() {
        for (int i = 0; i < chains.size(); i++) {
            ((PDBChain) chains.elementAt(i)).makeCaBondList();
        }
    }

    public PDBChain findChain(String id) {
        for (int i = 0; i < chains.size(); i++) {
            if (((PDBChain) chains.elementAt(i)).id.equals(id)) {
                return (PDBChain) chains.elementAt(i);
            }
        }

        return null;
    }

    public void setChargeColours() {
        for (int i = 0; i < chains.size(); i++) {
            ((PDBChain) chains.elementAt(i)).setChargeColours();
        }
    }

    public void setHydrophobicityColours() {
        for (int i = 0; i < chains.size(); i++) {
            ((PDBChain) chains.elementAt(i)).setHydrophobicityColours();
        }
    }

    public void setChainColours()
    {
       for (int i = 0; i < chains.size(); i++)
       {
            ((PDBChain) chains.elementAt(i)).setChainColours(
                     Color.getHSBColor(1.0f / (float)i, .4f, 1.0f)
                );
        }
    }
}
